import threading

try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object


class CrequestMiddleware(MiddlewareMixin):
    """
    Always have access to the current request
    """
    _request = {}

    def process_request(self, request):
        """
        Store request
        """
        self.__class__.set_request(request)

    def process_response(self, request, response):
        """
        Delete request
        """
        self.__class__.del_request()
        return response

    def process_exception(self, request, exception):
        """
        Delete request
        """
        self.__class__.del_request()

    @classmethod
    def get_request(cls, default=None):
        """
        Retrieve request
        """
        return cls._request.get(threading.current_thread(), default)

    @classmethod
    def set_request(cls, request):
        """
        Store request
        """
        cls._request[threading.current_thread()] = request

    @classmethod
    def del_request(cls):
        """
        Delete request
        """
        cls._request.pop(threading.current_thread(), None)



from setuptools import setup, find_packages


setup(
    name='django-crequest',
    version=".".join(map(str, __import__('crequest').__version__)),
    description='Middleware to make current request always available.',
    long_description=open('README.rst').read(),
    license=open('LICENSE').read(),
    author='Alireza Savand',
    author_email='alireza.savand@gmail.com',
    url='https://github.com/Alir3z4/django-crequest',
    packages=find_packages(exclude=['django_crequest']),
    install_requires=['Django>=1.2'],
    keywords=[
        'django',
        'request',
        'web'
    ],
    platforms='OS Independent',
    provides=['crequest',],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Development Status :: 6 - Mature",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries",
    ],
)
